@@ -1,7 +1,6 @@
 /* Update a tar archive.
 
-   Copyright 1988, 1992, 1994, 1996-1997, 1999-2001, 2003-2005, 2007,
-   2010, 2013-2014, 2016-2017 Free Software Foundation, Inc.
+   Copyright 1988-2019 Free Software Foundation, Inc.
 
    This file is part of GNU tar.
 
@@ -111,7 +110,7 @@ update_archive (void)
 
   name_gather ();
   open_archive (ACCESS_UPDATE);
-  buffer_write_global_xheader ();
+  xheader_forbid_global ();
 
   while (!found_end)
     {
