@@ -1,7 +1,6 @@
 /* POSIX extended headers for tar.
 
-   Copyright (C) 2003-2007, 2009-2010, 2012-2014, 2016 Free Software
-   Foundation, Inc.
+   Copyright (C) 2003-2019 Free Software Foundation, Inc.
 
    This file is part of GNU tar.
 
@@ -185,6 +184,9 @@ xheader_set_keyword_equal (char *kw, char *eq)
   bool global = true;
   char *p = eq;
 
+  if (eq == kw)
+    USAGE_ERROR ((0, 0, _("Malformed pax option: %s"), quote (kw)));
+    
   if (eq[-1] == ':')
     {
       p--;
@@ -255,7 +257,7 @@ char *
 xheader_format_name (struct tar_stat_info *st, const char *fmt, size_t n)
 {
   char *buf;
-  size_t len = strlen (fmt);
+  size_t len;
   char *q;
   const char *p;
   char *dirp = NULL;
@@ -266,43 +268,51 @@ xheader_format_name (struct tar_stat_info *st, const char *fmt, size_t n)
   char nbuf[UINTMAX_STRSIZE_BOUND];
   char const *nptr = NULL;
 
-  for (p = fmt; *p && (p = strchr (p, '%')); )
+  len = 0;
+  for (p = fmt; *p; p++)
     {
-      switch (p[1])
+      if (*p == '%' && p[1])
 	{
-	case '%':
-	  len--;
-	  break;
-
-	case 'd':
-	  if (st)
+	  switch (*++p)
 	    {
-	      if (!dirp)
-		dirp = dir_name (st->orig_file_name);
-	      dir = safer_name_suffix (dirp, false, absolute_names_option);
-	      len += strlen (dir) - 2;
+	    case '%':
+	      len++;
+	      break;
+
+	    case 'd':
+	      if (st)
+		{
+		  if (!dirp)
+		    dirp = dir_name (st->orig_file_name);
+		  dir = safer_name_suffix (dirp, false, absolute_names_option);
+		  len += strlen (dir);
+		}
+	      break;
+
+	    case 'f':
+	      if (st)
+		{
+		  base = last_component (st->orig_file_name);
+		  len += strlen (base);
+		}
+	      break;
+
+	    case 'p':
+	      pptr = umaxtostr (getpid (), pidbuf);
+	      len += pidbuf + sizeof pidbuf - 1 - pptr;
+	      break;
+
+	    case 'n':
+	      nptr = umaxtostr (n, nbuf);
+	      len += nbuf + sizeof nbuf - 1 - nptr;
+	      break;
+	      
+	    default:
+	      len += 2;
 	    }
-	  break;
-
-	case 'f':
-	  if (st)
-	    {
-	      base = last_component (st->orig_file_name);
-	      len += strlen (base) - 2;
-	    }
-	  break;
-
-	case 'p':
-	  pptr = umaxtostr (getpid (), pidbuf);
-	  len += pidbuf + sizeof pidbuf - 1 - pptr - 2;
-	  break;
-
-	case 'n':
-	  nptr = umaxtostr (n, nbuf);
-	  len += nbuf + sizeof nbuf - 1 - nptr - 2;
-	  break;
 	}
-      p++;
+      else
+	len++;
     }
 
   buf = xmalloc (len + 1);
@@ -458,6 +468,14 @@ xheader_write_global (struct xheader *xhdr)
     }
 }
 
+/* Forbid modifications of the global extended header */
+void
+xheader_forbid_global (void)
+{
+  if (keyword_global_override_list)
+    USAGE_ERROR ((0, 0, _("can't update global extended header record")));
+}
+
 void
 xheader_xattr_init (struct tar_stat_info *st)
 {
@@ -1290,15 +1308,33 @@ path_coder (struct tar_stat_info const *st, char const *keyword,
   code_string (st->file_name, keyword, xhdr);
 }
 
+static void
+raw_path_decoder (struct tar_stat_info *st, char const *arg)
+{
+  decode_string (&st->orig_file_name, arg);
+  decode_string (&st->file_name, arg);
+  st->had_trailing_slash = strip_trailing_slashes (st->file_name);
+}
+
+
 static void
 path_decoder (struct tar_stat_info *st,
 	      char const *keyword __attribute__((unused)),
 	      char const *arg,
 	      size_t size __attribute__((unused)))
 {
-  decode_string (&st->orig_file_name, arg);
-  decode_string (&st->file_name, arg);
-  st->had_trailing_slash = strip_trailing_slashes (st->file_name);
+  if (! st->sparse_name_done)
+    raw_path_decoder (st, arg);
+}
+
+static void
+sparse_path_decoder (struct tar_stat_info *st,
+                     char const *keyword __attribute__((unused)),
+                     char const *arg,
+                     size_t size __attribute__((unused)))
+{
+  st->sparse_name_done = true;
+  raw_path_decoder (st, arg);
 }
 
 static void
@@ -1371,7 +1407,7 @@ sparse_size_decoder (struct tar_stat_info *st,
   uintmax_t u;
   if (decode_num (&u, arg, TYPE_MAXIMUM (off_t), keyword))
     {
-      st->real_size_set = 1;
+      st->real_size_set = true;
       st->real_size = u;
     }
 }
@@ -1730,7 +1766,7 @@ struct xhdr_tab const xhdr_tab[] = {
   { "uname",    uname_coder,    uname_decoder,    0, false },
 
   /* Sparse file handling */
-  { "GNU.sparse.name",       path_coder, path_decoder,
+  { "GNU.sparse.name",       path_coder, sparse_path_decoder,
     XHDR_PROTECTED, false },
   { "GNU.sparse.major",      sparse_major_coder, sparse_major_decoder,
     XHDR_PROTECTED, false },
